@@ -6,7 +6,7 @@ package Excel::Writer::XLSX::Package::Styles;
 #
 # Used in conjunction with Excel::Writer::XLSX
 #
-# Copyright 2000-2015, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2023, John McNamara, jmcnamara@cpan.org
 #
 # Documentation after __END__
 #
@@ -20,7 +20,7 @@ use Carp;
 use Excel::Writer::XLSX::Package::XMLwriter;
 
 our @ISA     = qw(Excel::Writer::XLSX::Package::XMLwriter);
-our $VERSION = '0.86';
+our $VERSION = '1.10';
 
 
 ###############################################################################
@@ -42,14 +42,17 @@ sub new {
     my $fh    = shift;
     my $self  = Excel::Writer::XLSX::Package::XMLwriter->new( $fh );
 
-    $self->{_xf_formats}       = undef;
-    $self->{_palette}          = [];
-    $self->{_font_count}       = 0;
-    $self->{_num_format_count} = 0;
-    $self->{_border_count}     = 0;
-    $self->{_fill_count}       = 0;
-    $self->{_custom_colors}    = [];
-    $self->{_dxf_formats}      = [];
+    $self->{_xf_formats}         = undef;
+    $self->{_palette}            = [];
+    $self->{_font_count}         = 0;
+    $self->{_num_format_count}   = 0;
+    $self->{_border_count}       = 0;
+    $self->{_fill_count}         = 0;
+    $self->{_custom_colors}      = [];
+    $self->{_dxf_formats}        = [];
+    $self->{_has_hyperlink}      = 0;
+    $self->{_hyperlink_font_id}  = 0;
+    $self->{_has_comments}       = 0;
 
     bless $self, $class;
 
@@ -120,14 +123,15 @@ sub _set_style_properties {
 
     my $self = shift;
 
-    $self->{_xf_formats}       = shift;
-    $self->{_palette}          = shift;
-    $self->{_font_count}       = shift;
-    $self->{_num_format_count} = shift;
-    $self->{_border_count}     = shift;
-    $self->{_fill_count}       = shift;
-    $self->{_custom_colors}    = shift;
-    $self->{_dxf_formats}      = shift;
+    $self->{_xf_formats}         = shift;
+    $self->{_palette}            = shift;
+    $self->{_font_count}         = shift;
+    $self->{_num_format_count}   = shift;
+    $self->{_border_count}       = shift;
+    $self->{_fill_count}         = shift;
+    $self->{_custom_colors}      = shift;
+    $self->{_dxf_formats}        = shift;
+    $self->{_has_comments}       = shift;
 }
 
 
@@ -301,6 +305,11 @@ sub _write_fonts {
     my $self  = shift;
     my $count = $self->{_font_count};
 
+    if ( $self->{_has_comments} ) {
+        # Add an extra font for comments.
+        $count++;
+    }
+
     my @attributes = ( 'count' => $count );
 
     $self->xml_start_tag( 'fonts', @attributes );
@@ -310,6 +319,10 @@ sub _write_fonts {
         $self->_write_font( $format ) if $format->{_has_font};
     }
 
+    if ( $self->{_has_comments} ) {
+        $self->_write_comment_font();
+    }
+
     $self->xml_end_tag( 'fonts' );
 }
 
@@ -376,6 +389,10 @@ sub _write_font {
             $self->xml_empty_tag( 'family', 'val', $format->{_font_family} );
         }
 
+        if ($format->{_font_charset}) {
+            $self->xml_empty_tag( 'charset', 'val', $format->{_font_charset} );
+        }
+
         if ( $format->{_font} eq 'Calibri' && !$format->{_hyperlink} ) {
             $self->xml_empty_tag(
 
@@ -383,11 +400,38 @@ sub _write_font {
                 'val' => $format->{_font_scheme}
             );
         }
+
+        if ( $format->{_hyperlink} ) {
+            $self->{_has_hyperlink} = 1;
+
+            if ( !$self->{_hyperlink_font_id} ) {
+                $self->{_hyperlink_font_id} = $format->{_font_index};
+            }
+        }
     }
 
     $self->xml_end_tag( 'font' );
 }
 
+##############################################################################
+#
+# _write_comment_font()
+#
+# Write the <font> element used for comments.
+#
+sub _write_comment_font {
+
+    my $self = shift;
+
+    $self->xml_start_tag( 'font' );
+
+    $self->xml_empty_tag( 'sz', 'val', 8 );
+    $self->_write_color( 'indexed' => 81 );
+    $self->xml_empty_tag( 'name',   'val', 'Tahoma' );
+    $self->xml_empty_tag( 'family', 'val', 2 );
+
+    $self->xml_end_tag( 'font' );
+}
 
 ###############################################################################
 #
@@ -548,6 +592,11 @@ sub _write_fill {
 
     );
 
+    # Special handling for pattern only case.
+    if ( !$fg_color && !$bg_color && $format->{_pattern} ) {
+        $self->_write_default_fill( $patterns[ $format->{_pattern} ] );
+        return;
+    }
 
     $self->xml_start_tag( 'fill' );
 
@@ -574,7 +623,7 @@ sub _write_fill {
         $self->xml_empty_tag( 'bgColor', 'rgb' => $bg_color );
     }
     else {
-        if ( !$dxf_format ) {
+        if ( !$dxf_format && $format->{_pattern} <= 1) {
             $self->xml_empty_tag( 'bgColor', 'indexed' => 64 );
         }
     }
@@ -755,12 +804,20 @@ sub _write_cell_style_xfs {
     my $self  = shift;
     my $count = 1;
 
+    if ( $self->{_has_hyperlink} ) {
+        $count = 2;
+    }
+
     my @attributes = ( 'count' => $count );
 
     $self->xml_start_tag( 'cellStyleXfs', @attributes );
 
     # Write the style_xf element.
-    $self->_write_style_xf();
+    $self->_write_style_xf( 0, 0 );
+
+    if ( $self->{_has_hyperlink} ) {
+        $self->_write_style_xf( 1, $self->{_hyperlink_font_id} );
+    }
 
     $self->xml_end_tag( 'cellStyleXfs' );
 }
@@ -776,16 +833,8 @@ sub _write_cell_xfs {
 
     my $self    = shift;
     my @formats = @{ $self->{_xf_formats} };
+    my $count   = scalar @formats;
 
-    # Workaround for when the last format is used for the comment font
-    # and shouldn't be used for cellXfs.
-    my $last_format = $formats[-1];
-
-    if ( $last_format->{_font_only} ) {
-        pop @formats;
-    }
-
-    my $count = scalar @formats;
     my @attributes = ( 'count' => $count );
 
     $self->xml_start_tag( 'cellXfs', @attributes );
@@ -807,11 +856,12 @@ sub _write_cell_xfs {
 #
 sub _write_style_xf {
 
-    my $self       = shift;
-    my $num_fmt_id = 0;
-    my $font_id    = 0;
-    my $fill_id    = 0;
-    my $border_id  = 0;
+    my $self          = shift;
+    my $has_hyperlink = shift;
+    my $font_id       = shift;
+    my $num_fmt_id    = 0;
+    my $fill_id       = 0;
+    my $border_id     = 0;
 
     my @attributes = (
         'numFmtId' => $num_fmt_id,
@@ -820,7 +870,21 @@ sub _write_style_xf {
         'borderId' => $border_id,
     );
 
-    $self->xml_empty_tag( 'xf', @attributes );
+    if ( $has_hyperlink ) {
+        push @attributes, ( 'applyNumberFormat' => 0 );
+        push @attributes, ( 'applyFill'         => 0 );
+        push @attributes, ( 'applyBorder'       => 0 );
+        push @attributes, ( 'applyAlignment'    => 0 );
+        push @attributes, ( 'applyProtection'   => 0 );
+
+        $self->xml_start_tag( 'xf', @attributes );
+        $self->xml_empty_tag( 'alignment',  ( 'vertical', 'top' ) );
+        $self->xml_empty_tag( 'protection', ( 'locked',   0 ) );
+        $self->xml_end_tag( 'xf' );
+    }
+    else {
+        $self->xml_empty_tag( 'xf', @attributes );
+    }
 }
 
 
@@ -838,7 +902,7 @@ sub _write_xf {
     my $font_id     = $format->{_font_index};
     my $fill_id     = $format->{_fill_index};
     my $border_id   = $format->{_border_index};
-    my $xf_id       = 0;
+    my $xf_id       = $format->{_xf_id};
     my $has_align   = 0;
     my $has_protect = 0;
 
@@ -856,7 +920,7 @@ sub _write_xf {
     }
 
     # Add applyFont attribute if XF format uses a font element.
-    if ( $format->{_font_index} > 0 ) {
+    if ( $format->{_font_index} > 0 && !$format->{_hyperlink} ) {
         push @attributes, ( 'applyFont' => 1 );
     }
 
@@ -877,16 +941,19 @@ sub _write_xf {
     $has_align = 1 if $apply_align && @align;
 
     # We can also have applyAlignment without a sub-element.
-    if ( $apply_align ) {
+    if ( $apply_align || $format->{_hyperlink} ) {
         push @attributes, ( 'applyAlignment' => 1 );
     }
 
     # Check for cell protection properties.
     my @protection = $format->get_protection_properties();
 
-    if ( @protection ) {
+    if ( @protection || $format->{_hyperlink} ) {
         push @attributes, ( 'applyProtection' => 1 );
-        $has_protect = 1;
+
+        if ( !$format->{_hyperlink} ) {
+            $has_protect = 1;
+        }
     }
 
     # Write XF with sub-elements if required.
@@ -913,12 +980,20 @@ sub _write_cell_styles {
     my $self  = shift;
     my $count = 1;
 
+    if ( $self->{_has_hyperlink} ) {
+        $count = 2;
+    }
+
     my @attributes = ( 'count' => $count );
 
     $self->xml_start_tag( 'cellStyles', @attributes );
 
     # Write the cellStyle element.
-    $self->_write_cell_style();
+    if ( $self->{_has_hyperlink} ) {
+        $self->_write_cell_style('Hyperlink', 1, 8);
+    }
+
+    $self->_write_cell_style('Normal', 0, 0);
 
     $self->xml_end_tag( 'cellStyles' );
 }
@@ -933,9 +1008,9 @@ sub _write_cell_styles {
 sub _write_cell_style {
 
     my $self       = shift;
-    my $name       = 'Normal';
-    my $xf_id      = 0;
-    my $builtin_id = 0;
+    my $name       = shift;
+    my $xf_id      = shift;
+    my $builtin_id = shift;
 
     my @attributes = (
         'name'      => $name,
@@ -1118,7 +1193,7 @@ John McNamara jmcnamara@cpan.org
 
 =head1 COPYRIGHT
 
-(c) MM-MMXV, John McNamara.
+(c) MM-MMXXIII, John McNamara.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
 
